/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.datasource.util;

import static org.junit.Assert.assertEquals;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashMap;

import org.junit.Test;

import com.aquenos.epics.jackie.diirt.datasource.util.SimpleJsonGenerator;

/**
 * Tests for the {@link SimpleJsonGenerator}. Due to whitespace and the
 * possibility to escape Unicode code-points in strings, there is an infinite
 * number of JSON documents that are equivalent. The tests in this class assume
 * that the simplest form of serialization is used (no whitespace, only escape
 * those characters in strings that must be escaped). As the tested generator
 * implements the serialization like this, this should lead to correct results.
 * 
 * @author Sebastian Marsching
 */
public class SimpleJsonGeneratorTest {

    private static String generate(Object object) {
        return SimpleJsonGenerator.generate(object);
    }

    private static void testNumber(Number number) {
        // This only works for finite numbers.
        assertEquals(number.toString(), generate(number));
    }

    /**
     * Tests that JSON arrays are serialized correctly.
     */
    @Test
    public void arrays() {
        assertEquals("[]", generate(Collections.emptyList()));
        assertEquals("[true]", generate(Collections.singletonList(true)));
        assertEquals("[\"abc\"]", generate(Collections.singletonList("abc")));
        assertEquals("[null]", generate(Collections.singletonList(null)));
        assertEquals("[\"abc\",null,\"def\",false]",
                generate(Arrays.asList("abc", null, "def", false)));
    }

    /**
     * Tests that the boolean <code>false</code> is serialized correctly.
     */
    @Test
    public void falseValue() {
        assertEquals("false", generate(Boolean.FALSE));
    }

    /**
     * Tests that the value <code>null</code> is serialized correctly.
     */
    @Test
    public void nullValue() {
        assertEquals("null", generate(null));
    }

    /**
     * Tests that numbers are serialized correctly.
     */
    @Test
    public void numberValues() {
        testNumber(5.384);
        testNumber(-7.384);
        testNumber(2.0e-3);
        testNumber(-5e22);
        testNumber(1234567890L);
        testNumber((short) 0);
        testNumber(0.00);
        testNumber(new BigInteger("-48"));
        testNumber(new BigDecimal("1e50000"));
        assertEquals("\"NaN\"", generate(Double.NaN));
        assertEquals("\"-Infinity\"", generate(Double.NEGATIVE_INFINITY));
        assertEquals("\"Infinity\"", generate(Double.POSITIVE_INFINITY));
        assertEquals("\"NaN\"", generate(Float.NaN));
        assertEquals("\"-Infinity\"", generate(Float.NEGATIVE_INFINITY));
        assertEquals("\"Infinity\"", generate(Float.POSITIVE_INFINITY));
    }

    /**
     * Tests that maps objects are correctly serialized to JSON objects.
     */
    @Test
    public void objects() {
        assertEquals("{}", generate(Collections.emptyMap()));
        assertEquals("{\"boolean\":true}",
                generate(Collections.singletonMap("boolean", true)));
        assertEquals("{\"string\":\"abc\"}",
                generate(Collections.singletonMap("string", "abc")));
        assertEquals("{\"null\":null}",
                generate(Collections.singletonMap(null, null)));
        assertEquals("{\"5\":123}", generate(Collections.singletonMap(5, 123)));
        assertEquals("{\"nested\":{\"test\":true}}",
                generate(Collections.singletonMap("nested",
                        Collections.singletonMap("test", true))));
        LinkedHashMap<String, Object> testMap = new LinkedHashMap<String, Object>();
        testMap.put("k1", "abc");
        testMap.put("k2", null);
        testMap.put("k3", "def");
        testMap.put("k4", false);
        String testJson = "{\"k1\":\"abc\",\"k2\":null,\"k3\":\"def\",\"k4\":false}";
        assertEquals(testJson, generate(testMap));
    }

    /**
     * Tests that JSON strings are serialized correctly.
     */
    @Test
    public void stringValues() {
        assertEquals("\"a\\\"b\\\\c\\näöü\"", generate("a\"b\\c\näöü"));
        assertEquals("\"\"", generate(""));
        assertEquals("\"\\\"\"", generate("\""));
        assertEquals("\" \\n\\u0005\"", generate(" \n\u0005"));
    }

    /**
     * Tests that the boolean <code>true</code> is serialized correctly.
     */
    @Test
    public void trueValue() {
        assertEquals("true", generate(Boolean.TRUE));
    }

}
