/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.vtype;

import java.nio.FloatBuffer;
import java.time.Instant;

import org.diirt.util.array.ListFloat;
import org.diirt.vtype.VFloatArray;
import org.diirt.vtype.VTypeToString;

import com.aquenos.epics.jackie.common.value.ChannelAccessControlsFloat;
import com.aquenos.epics.jackie.common.value.ChannelAccessTimeFloat;

/**
 * {@link VFloatArray} implementation that is backed by a
 * {@link ChannelAccessControlsFloat} and {@link ChannelAccessTimeFloat}.
 * 
 * @author Sebastian Marsching
 */
public final class JackieVFloatArray extends
        JackieVNumberArray<ChannelAccessControlsFloat, ChannelAccessTimeFloat>
        implements VFloatArray {

    /**
     * <p>
     * Creates a value backed by the specified {@code controlsValue} and
     * {@code timeValue}. The {@code controlsValue} is used for the meta-data
     * (engineering units, limits, etc.) and the {@code timeValue} is used for
     * the alarm status, time-stamp, and value.
     * </p>
     * 
     * <p>
     * If {@code disconnectTime} is not <code>null</code>, it is used as the
     * time-stamp and the alarm severity is set to {@code UNDEFINED}.
     * </p>
     * 
     * @param controlsValue
     *            value used for meta-data (engineering units, limits, etc.).
     * @param timeValue
     *            value used for alarm status, time-stamp, and value.
     * @param disconnectTime
     *            time at which the channel was disconnected. Usually this
     *            should be <code>null</code>. If it is not <code>null</code>,
     *            the alarm severity is set to {@code UNDEFINED} and this
     *            parameter is used as the time-stamp (instead of using the
     *            time-stamp from the {@code timeValue}.
     * @param honorZeroPrecision
     *            defines how the value is formatted if the
     *            {@code controlsValue}'s precision is zero. If
     *            <code>true</code> a precision of zero means that no decimal
     *            digits shall be printed. If <code>false</code>, a precision of
     *            zero means that a default number of decimal digits shall be
     *            printed. This parameter is only relevant for the return value
     *            of the {@link #getFormat()} method.
     */
    public JackieVFloatArray(ChannelAccessControlsFloat controlsValue,
            ChannelAccessTimeFloat timeValue, Instant disconnectTime,
            boolean honorZeroPrecision) {
        super(controlsValue, timeValue, disconnectTime, honorZeroPrecision);
    }

    @Override
    public ListFloat getData() {
        FloatBuffer buffer = timeValue.getValue();
        return new ListFloat() {
            @Override
            public int size() {
                return buffer.remaining();
            }

            @Override
            public float getFloat(int index) {
                return buffer.get(index);
            }
        };
    }

    @Override
    public String toString() {
        return VTypeToString.toString(this);
    }

}
